#!/bin/sh -x

PATH=/sbin:/usr/sbin:/bin:/usr/bin

COMMAND=${1}
CLONEMAC=${2}
BRACC=1
WIFIMODE=AD-HOC

if [ -r /etc/init.d/bridge.conf ] ; then
	. /etc/init.d/bridge.conf
fi

if [ -e /etc/init.d/ipconfig ] ; then
	. /etc/init.d/ipconfig
fi

PATH_BRNFCNF=/proc/sys/net/bridge

br_start_direct() {
	echo -n "Setting bridge:                         " > /dev/console
	echo "br0 < eth0 - ath0 >" > /dev/console

	#
	# Convert WifiMode
	#
	case "${WIFIMODE}" in
	'INFRASTRUCTURE'|'AUTO')
		modestr="sta" ;;
	'AD-HOC' | *)
		modestr="ibss" ;;
	esac

	/etc/init.d/loadar6k.sh ${modestr} ${CLONEMAC} 1

	sleep 1
	if [ -e /sys/class/net/ath0 ] ; then
		ifconfig eth0 0.0.0.0
		brctl addif br0 eth0
		brctl addif br0 ath0
		if [ ${BRACC} -eq 1 ] ; then
			/etc/init.d/startip.sh br0
		fi

		if [ -e /etc/wpa_supplicant.conf ]; then
			cp /etc/wpa_supplicant.conf /tmp/wpa_supplicant.conf
			echo "wpa_supplicant -i ath0 -B -D ar6003 -c /etc/wpa_supplicant.conf -W -M ${CLONEMAC} -b br0 -s" > /dev/console
# for debug mode
#			wpa_supplicant -i ath0 -B -D ar6003 -c /tmp/wpa_supplicant.conf -W -M ${CLONEMAC} -b br0 -f /tmp/wpa.log -ddd
			wpa_supplicant -i ath0 -B -D ar6003 -c /tmp/wpa_supplicant.conf -W -M ${CLONEMAC} -b br0 -s

			WPA_COMMENT="`tail -n 1 /tmp/wpa_supplicant.conf`"
			echo "Start wpa_supplicant : ${WPA_COMMENT}" > /dev/console
		fi
	else
		echo "ath0 I/F is not ready" > /dev/console
	fi

	# Disable bridge netfilter
#	FILES=`ls ${PATH_BRNFCNF}`
#	for FILE in ${FILES}
#	do
#		if [ -w "${PATH_BRNFCNF}/${FILE}" ]; then
#			echo "0" > ${PATH_BRNFCNF}/${FILE}
#		fi
#	done

	# Start DHCP server
	if [ -x /etc/init.d/udhcpd.sh ]; then
		/etc/init.d/udhcpd.sh br0 sysstart
	fi
}

br_start() {
	# waits for the end of a rcW process. 
	while [ ! -e /tmp/init_complete ]
	do
		echo "<BR Wait>" > /dev/console
		sleep 1
	done

	br_start_direct

	# Startup the protocol depending on an interface
	#
	# Start JCP server
	#
	if [ -x /usr/sbin/jcpd ] ; then
		/usr/sbin/jcpd -i br0
	fi

	#
	# Start FLDP server
	#
	if [ -x /usr/sbin/fldpd ] ; then
		/usr/sbin/fldpd -D -i br0 
	fi
}

br_stop() {
	echo "Stop bridge" > /dev/console
	ifconfig ath0 down
	wmiconfig -i ath0 --wlan disable
}

case "${COMMAND}" in
'start_direct')
	br_start_direct
	;;
'start')
	br_start
	;;
'stop')
	br_stop
	;;
*)
	echo "usage $0 start|stop"
	;;
esac
