#!/bin/sh


mode="sta"
brmode="0"
macaddr="00:00:00:00:00:00"

# Mode 
#  sta : Station
# ibss : Adhoc
#   ap : AccessPoint
if [ -n "${1}" ] ; then
	mode=${1}
fi

# ath0 I/F MacAddress
# "xx:xx:xx:xx:xx:xx"
if [ -n "${2}" ] ; then
	macaddr=${2}
fi
# brmode : Station Bridge Mode Enable/Disable
# 1 : enable
# 2 : disable
if [ -n "${3}" ] ; then
	brmode=${3}
fi
fakeaddr=`ubootenv ethaddr`
#fakeaddr=00:80:92:11:11:11

echo "mode=${mode} mac=${macaddr} brmode=${brmode}"


# Define interface
ifc=ath0

# Fetch configuration file
if [ -e /etc/init.d/ipconfig ] ; then
   . /etc/init.d/ipconfig
fi

# Get region domain
if [ -z "${REGDOM}" ] ; then
   # Default to WWR if missing
   REGDOM=0x406A

   REGDOM_ENV=`ubootenv regdomain`

   case "$REGDOM_ENV" in
   "WWR")
         REGDOM=0x406A
         ;;
   "US")
         # Use World Wide Roaming when "US".
         #REGDOM=0x8348
         REGDOM=0x406A
         ;;
   "DE")
         REGDOM=0x8114
         ;;
   "JP")
         REGDOM=0x8188
         ;;
   esac
fi

echo "Region domain $REGDOM" > /dev/console

RADIO=`ubootenv radio`

if [ $RADIO = "sxsdman" ] ; then
   boardData=calData_SX-SDMAN.bin
else
   boardData=calData_SX-SDMGN.bin
fi

# Define location of firmware and data files
fw_dir=/lib/modules/firmware

#
# Load wireless kernel modules first since they take time to
# initialize
#
if [ ${brmode} = "1" ] ; then
	insmod /lib/modules/ar6000.ko ifname=$ifc bmienable=1 debugdriver=0 busspeedlow=0 devmode=${mode} stabrmode=1 fakemacaddr=${fakeaddr}
else
	insmod /lib/modules/ar6000.ko ifname=$ifc bmienable=1 debugdriver=0 busspeedlow=0 devmode=${mode}
fi

# Get target info (also waits for device to be ready)
bmiloader -i $ifc --quiet --info >/dev/null

#
# Temporarily disable System Sleep
#
old_options=`bmiloader -i $ifc --quiet --set --address=0x180c0 --or=8`
old_sleep=`bmiloader -i $ifc --quiet --set --address=0x40c4 --or=1`

#
# Clock setup.
#
bmiloader -i $ifc --quiet --set --address=0x1c284 --param=0xF9104001
bmiloader -i $ifc --quiet --set --address=0x4020 --param=1

#
# Lower pad drive strengths; Temporary(?) WAR for SDIO CRC issue
#
bmiloader -i $ifc --quiet --set --address=0x14050 --param=0x20
bmiloader -i $ifc --quiet --set --address=0x14054 --param=0x20
bmiloader -i $ifc --quiet --set --address=0x14058 --param=0x20
bmiloader -i $ifc --quiet --set --address=0x1405c --param=0x20

# It is the Host's responsibility to set hi_refclk_hz
# according to the actual speed of the reference clock.
bmiloader -i $ifc --quiet --set --address=0x40e0 --param=0x100000

if [ ${macaddr} = "00:00:00:00:00:00" ] ; then
	echo "use default MAC" > /dev/console
	#
	# Part 1: Load board data
	#
	echo "AR6K: loading board data from " $fw_dir/$boardData > /dev/console
	eeprom --transfer --setreg=$REGDOM --file=$fw_dir/$boardData --interface=$ifc
	#
	# Part 2: Load OTP
	#
	bmiloader -i $ifc --write --file=$fw_dir/otp.bin
	bmiloader -i $ifc --execute --address=0x945d00 --param=2
else
	echo "use user MAC ${macaddr}" > /dev/console
	#
	# Part 1: Load board data
	#
	eeprom --transfer --setmac=${macaddr} --setreg=$REGDOM --file=$fw_dir/$boardData --interface=$ifc
	#
	# Part 2: Load OTP
	#
	bmiloader -i $ifc --write --file=$fw_dir/otp.bin
	bmiloader -i $ifc --execute --address=0x945d00 --param=3
fi

#
# Part 3: athwlan RAM firmware
#
fw_bin=$fw_dir/athwlan.bin
athtcmd_bin=$fw_dir/athtcmd_ram.bin
if [ -f $athtcmd_bin ] ; then
	fw_bin=$athtcmd_bin
fi
echo "AR6K: loading firmware from ${fw_bin}" > /dev/console
bmiloader -i $ifc --write --file=$fw_bin

#
# Part 4: RAM DataSets
#
LOADADDR=0x57ff28
bmiloader -i $ifc --write --address=${LOADADDR} --file=$fw_dir/data.patch.hw3_0.bin

# Set begin address
bmiloader -i $ifc --write --address=5506584 --param=${LOADADDR}

# Configure GPIO 22 as BT reset pin
bmiloader -i $ifc --set --address=0x5406a4 --param=0x16

# Configure UART
bmiloader -i $ifc --set --address=0x540680 --param=8

# Restore System Sleep on AR6003
bmiloader -i $ifc --quiet --set --address=0x40c4 --param=$old_sleep > /dev/null
bmiloader -i $ifc --quiet --set --address=0x180c0 --param=$old_options > /dev/null

# silex addition
bmiloader -i $ifc --quiet --write --address=5506764 --param=2

# Leave BMI now and start the WLAN driver.
bmiloader -i $ifc --quiet --done

# Tell kernel the interface is up
ifconfig $ifc up

exit 0
