#!/bin/sh -x

#
# Firmware update script
# Model: SX-570/580
# Created: 2013.08.21
#
# $1 is the name of the tarball
#
TB=$1

# Remember where we started
WD=`pwd`

echo $WD

if [ -z "$TB" ] ; then
   echo -e "\nUsage: update.sh tarball" > /dev/console
   exit 1
fi

echo -e "\nFirmware update in progress. Do NOT power off!" > /dev/console

# Extract blinkled and all md5 files
tar -jxf $TB blinkled '*.md5' '*.sum'

if [ ! -e blinkled ] ; then
   exit 1
fi

# Begin blinking the LED during the update
./blinkled
rm ./blinkled

#
# Kill off all applications that we will be updating
#
killall -q bb6k3
killall -q garpd
killall -q console
killall -q thttpd
killall -q snmpd
killall -q inetd
killall -q udhcpc
killall -q udhcpd
killall -q jcpd
killall -q ntpd
killall -q wpa_supplicant
killall -q hostapd
killall -q fldpd
killall -q sxbrd
killall -q jcpd_uvl
killall -q inotifyd

#
# Update a symbolic link
# $1 = directory
# $2 = symbolic link
# $3 = target file
#
# If link does not exist or points to wrong file, it is re-created
#
updatelink() {
   if [ -e $1/$2 ] ; then
      cmp -s $1/$2 $1/$3
      if [ $? == 0 ] ; then
	# Link is correct - our job is done
	return 0
      fi
   fi
   # Delete incorrect link
   rm -f $1/$2
   cd $1
   # Create new link
   ln -s $3 $2
   cd $WD
}

#
# Fetch file if not already there
# Script normally prefetches multiple files ahead of time for speed
#
fetch() {
  if [ ! -e $1 ] ; then
    echo -e $1 " NOT FOUND" > /dev/console
    tar -jxf $WD/$TB $1
  fi
}

#
# Update a file
# $1 = directory
# $2 = target file
# $3 = symbolic link (optional)
# $4 = symbolic link (optional)
#
# If file does not exist or does not match new file, it is re-created
#
updatefile() {
#   echo -e "Updating " $1/$2 > /dev/console
   if [ -e $1/$2 ] ; then
      cd $1
      md5sum -c -s $WD/$2.md5
      if [ $? -ne 0 ] ; then
        cd $WD
        fetch $2
        cp -f $2 $1/$2
        echo -e $1/$2 " updated" > /dev/console
      fi
      cd $WD
   else
      fetch $2
      cp -f $2 $1/$2
      echo -e $1/$2 " installed" > /dev/console
   fi
   rm -f $2 $2.md5
   if [ -n "$3" ] ; then
      updatelink $1 $3 $2
   fi
   if [ -n "$4" ] ; then
      updatelink $1 $4 $2
   fi
}

# Fetch all the files at once - faster than one at a time

tar -jxf $TB \
	bb6k3 console garpd ubootenv sxbrd www-update.tar.bz2 \
	rcW rcD profile \*.sh dhcpscfg \
	bmiloader eeprom wmiconfig wpa_cli wpa_supplicant hostapd_cli hostapd_wps hostapd \
	\*.ko \*.bin \
	jcpd_uvl \
	fldpd sxcpsync sxbrd jcpd www-update.tar.bz2 busybox \
	busybox-bin-update.tar.bz2 busybox-usr-sbin-update.tar.bz2 \
	cpuclkgov.sh wpa_supplicant_null.conf hostapd_cli_action.sh \
	hostapd_cli_event.sh wpa_cli_action.sh wpa_action.sh \
	wps.stat syslog.sh wireless.conf \
	snmpd.conf snmpd snmpget snmpset libnetsnmpmibs.so.30.0.1 libnetsnmptrapd.so.30.0.1 \
	sxspsconf sxsysconf ssi ethtool

#
# update bb6
#
updatefile "/opt" "bb6k3"
updatefile "/usr/bin" "console"
updatefile "/usr/sbin" "garpd"
updatefile "/usr/sbin" "ubootenv"

#
# update base 
#
updatefile "/etc/init.d" "wpa_action.sh"
updatefile "/etc/init.d" "sxuptp_action.sh"
updatefile "/etc/init.d" "rcW"
updatefile "/etc/init.d" "rcD"
updatefile "/etc/init.d" "loadar6k.sh"
updatefile "/etc/init.d" "bridge.sh"
updatefile "/etc/init.d" "resetar6k.sh"
updatefile "/etc/init.d" "dhcpscfg"
updatefile "/etc/init.d" "udhcpd.sh"
updatefile "/etc" "profile"

updatefile "/etc/init.d" "cpuclkgov.sh"
if [ ! -e /etc/wpa_supplicant_null.conf ]; then
	updatefile "/etc" "wpa_supplicant_null.conf"
fi
updatefile "/etc/init.d" "hostapd_cli_action.sh"
updatefile "/etc/init.d" "hostapd_cli_event.sh"
updatefile "/etc/init.d" "wpa_cli_action.sh"

if [ ! -d /etc/silex ] ; then
	mkdir /etc/silex
fi
updatefile "/etc/silex"  "wps.stat"
updatefile "/etc/init.d" "syslog.sh"
updatefile "/etc" "wireless.conf"
updatefile "/etc" "snmpd.conf"

if [ ! -d /var/lib/misc ] ; then
	ln -sf ../../tmp /var/lib/misc 
fi

#
# update wlan driver
#
updatefile "/lib/modules" "ar6000.ko"
updatefile "/usr/sbin" "bmiloader"
updatefile "/usr/sbin" "eeprom"
updatefile "/usr/bin" "wmiconfig"
updatefile "/usr/bin" "wpa_cli"
updatefile "/usr/sbin" "wpa_supplicant"
updatefile "/usr/bin" "hostapd_cli"
updatefile "/usr/bin" "hostapd_wps"
updatefile "/usr/sbin" "hostapd"

updatefile "/lib/modules/firmware" "athwlan.bin"
updatefile "/lib/modules/firmware" "otp.bin"
updatefile "/lib/modules/firmware" "data.patch.hw3_0.bin"
updatefile "/lib/modules/firmware" "calData_SX-SDMAN.bin"
updatefile "/lib/modules/firmware" "calData_SX-SDMGN.bin"

#
# update kernel object
#
updatefile "/lib/modules" "bluetooth.ko"
updatefile "/lib/modules" "ip_tables.ko"
updatefile "/lib/modules" "ipt_REJECT.ko"
updatefile "/lib/modules" "iptable_filter.ko"
updatefile "/lib/modules" "mmc_block.ko"
updatefile "/lib/modules" "mmc_core.ko"
updatefile "/lib/modules" "mxs-mmc.ko"
updatefile "/lib/modules" "sx570_gpio.ko"
updatefile "/lib/modules" "x_tables.ko"
updatefile "/lib/modules" "xt_iprange.ko"

#
# update netsnmp
#
updatefile "/sbin" "snmpd"
updatefile "/lib" "libnetsnmpmibs.so.30.0.1"
updatefile "/lib" "libnetsnmptrapd.so.30.0.1" "libnetsnmptrapd.so.30" "libnetsnmptrapd.so"
updatefile "/usr/bin"  "snmpget"
updatefile "/usr/bin"  "snmpset"

#
# update sxuptp
#
updatefile "/usr/sbin" "jcpd_uvl"
updatefile "/lib/modules" "sxuptp_devfilter.ko"
updatefile "/lib/modules" "sxuptp.ko"
updatefile "/lib/modules" "sxuptp_driver.ko"
updatefile "/lib/modules" "sxuptp_wq.ko"

#
# update other 
#
updatefile "/usr/sbin" "fldpd"
updatefile "/usr/sbin" "sxcpsync"
updatefile "/usr/sbin" "sxbrd"
updatefile "/usr/sbin" "jcpd"
# sxutils
updatefile "/usr/sbin" "sxspsconf"
updatefile "/usr/sbin" "sxsysconf"

# sxssi
updatefile "/usr/share/www" "ssi"

# ethtool
updatefile "/usr/sbin" "ethtool"

# delete "default" usr and group (v2.00)
/bin/sed -i -e "/^default/d" /etc/shadow
/bin/sed -i -e "/^default/d" /etc/passwd
/bin/sed -i -e "/^default/d" /etc/group


#
# Add device files
#
if [ ! -e /dev/spidev2.0 ] ; then
	mknod -m 666 /dev/spidev2.0 c 153 0
	echo -e "/dev/spidev2.0 created" > /dev/console
fi

#
# update www files
#
rm -rf /usr/www/*
tar -C /usr -jxf www-update.tar.bz2
echo -e "/usr/www updated" > /dev/console


#
# update busybox
#
busybox_updatefile() {
#   echo -e "Updating " $1/$2 > /dev/console
	cd $1
	md5sum -c -s $WD/$2.md5
	if [ $? -ne 0 ] ; then
		cd $WD
		fetch $2
		$WD/$2 cp -f $2 $1/$2
		echo -e $1/$2 " updated" > /dev/console

		# update busybox symbolic links
		tar -C / -jxf busybox-bin-update.tar.bz2
		echo -e "/bin updated" > /dev/console

		tar -C /usr -jxf busybox-usr-sbin-update.tar.bz2
		echo -e "/usr/sbin updated" > /dev/console
	fi
	cd $WD
	rm -f $2 $2.md5
}

busybox_updatefile "/bin" "busybox"


#
# Now see if we need to update to the new kernel
#
#md5sum -c -s mtd2.sum
if [ true ] ; then
#if [ $? -ne 0 ] ; then
   tar -jxf $TB uImage
   #
   # Confirm we have a valid kernel before erasing the old one
   #
   md5sum -c -s uImage.md5
   if [ $? -eq 0 ] ; then
      echo -e "\nInstalling new kernel..." > /dev/console
      busybox flashcp -v uImage /dev/mtd2
      if [ $? -ne 0 ] ; then
	echo -e "\n!! Kernel installation failure !!" > /dev/console
	exit 0
      fi
      echo -e "\nNew kernel installed" > /dev/console
   else
	echo -e "\nNew kernel image not valid. Update aborted" > /dev/console
	reboot
   fi
else
	echo -e "\nKernel update already installed" > /dev/console
fi

#
# Flush flash changes and dismount file system
#
sync
sync
mount -r -o remount /

echo -e "\nFirmware update completed successfully. Restarting..." > /dev/console

#
# That's all folks!
#
reboot
